@@ -26,9 +26,7 @@
 #include "uuid/uuid.h"
 #include "journal.h"
 
-#ifdef CONFIG_JBD_DEBUG		/* Enabled by configure --enable-jfs-debug */
 static int bh_count = 0;
-#endif
 
 #if EXT2_FLAT_INCLUDES
 #include "blkid.h"
@@ -135,10 +133,8 @@ struct buffer_head *getblk(kdev_t kdev, unsigned long long blocknr,
 	if (retval)
 		return NULL;
 
-#ifdef CONFIG_JBD_DEBUG
 	if (journal_enable_debug >= 3)
 		bh_count++;
-#endif
 	jfs_debug(4, "getblk for block %llu (%d bytes)(total %d)\n",
 		  blocknr, blocksize, bh_count);
 
@@ -165,7 +161,8 @@ int sync_blockdev(kdev_t kdev)
 	return io_channel_flush(io) ? EIO : 0;
 }
 
-void ll_rw_block(int rw, int op_flags, int nr, struct buffer_head *bhp[])
+void ll_rw_block(int rw, int op_flags EXT2FS_ATTR((unused)), int nr,
+		 struct buffer_head *bhp[])
 {
 	errcode_t retval;
 	struct buffer_head *bh;
@@ -248,6 +245,8 @@ void wait_on_buffer(struct buffer_head *bh)
 
 static void ext2fs_clear_recover(ext2_filsys fs, int error)
 {
+	time_t s_mtime;
+
 	ext2fs_clear_feature_journal_needs_recovery(fs->super);
 
 	/* if we had an error doing journal recovery, we need a full fsck */
@@ -257,8 +256,9 @@ static void ext2fs_clear_recover(ext2_filsys fs, int error)
 	 * If we replayed the journal by definition the file system
 	 * was mounted since the last time it was checked
 	 */
-	if (fs->super->s_lastcheck >= fs->super->s_mtime)
-		fs->super->s_lastcheck = fs->super->s_mtime - 1;
+	s_mtime = ext2fs_get_tstamp(fs->super, s_mtime);
+	if (ext2fs_get_tstamp(fs->super, s_lastcheck) >= s_mtime)
+		ext2fs_set_tstamp(fs->super, s_lastcheck, s_mtime - 1);
 	ext2fs_mark_super_dirty(fs);
 }
 
@@ -772,6 +772,8 @@ static errcode_t recover_ext3_journal(ext2_filsys fs)
 		mark_buffer_dirty(journal->j_sb_buffer);
 	}
 
+	journal->j_tail_sequence = journal->j_transaction_sequence;
+
 errout:
 	jbd2_journal_destroy_revoke(journal);
 	jbd2_journal_destroy_revoke_record_cache();
@@ -790,6 +792,8 @@ errcode_t ext2fs_run_ext3_journal(ext2_filsys *fsp)
 	char *fsname;
 	int fsflags;
 	int fsblocksize;
+	char *save;
+	__u16 s_error_state;
 
 	if (!(fs->flags & EXT2_FLAG_RW))
 		return EXT2_ET_FILE_RO;
@@ -809,6 +813,12 @@ errcode_t ext2fs_run_ext3_journal(ext2_filsys *fsp)
 	if (stats && stats->bytes_written)
 		kbytes_written = stats->bytes_written >> 10;
 
+	save = malloc(EXT4_S_ERR_LEN);
+	if (save)
+		memcpy(save, ((char *) fs->super) + EXT4_S_ERR_START,
+		       EXT4_S_ERR_LEN);
+	s_error_state = fs->super->s_state & EXT2_ERROR_FS;
+
 	ext2fs_mmp_stop(fs);
 	fsname = fs->device_name;
 	fs->device_name = NULL;
@@ -819,11 +829,15 @@ errcode_t ext2fs_run_ext3_journal(ext2_filsys *fsp)
 	retval = ext2fs_open(fsname, fsflags, 0, fsblocksize, io_ptr, fsp);
 	ext2fs_free_mem(&fsname);
 	if (retval)
-		return retval;
+		goto outfree;
 
 	fs = *fsp;
 	fs->flags |= EXT2_FLAG_MASTER_SB_ONLY;
 	fs->super->s_kbytes_written += kbytes_written;
+	fs->super->s_state |= s_error_state;
+	if (save)
+		memcpy(((char *) fs->super) + EXT4_S_ERR_START, save,
+		       EXT4_S_ERR_LEN);
 
 	/* Set the superblock flags */
 	ext2fs_clear_recover(fs, recover_retval != 0);
@@ -833,6 +847,9 @@ errcode_t ext2fs_run_ext3_journal(ext2_filsys *fsp)
 	 * the EXT2_ERROR_FS flag in the fs superblock if needed.
 	 */
 	retval = ext2fs_check_ext3_journal(fs);
+
+outfree:
+	free(save);
 	return retval ? retval : recover_retval;
 }
 
